@@ -2,7 +2,7 @@
 # -*- coding: utf8 -*-
 
 # ============================================================================
-#  Copyright (c) 2014-2017 nexB Inc. http://www.nexb.com/ - All rights reserved.
+#  Copyright (c) 2014-2018 nexB Inc. http://www.nexb.com/ - All rights reserved.
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at
@@ -518,12 +518,11 @@ class AboutTest(unittest.TestCase):
         file_path1 = posixpath.join(posixpath.dirname(test_file), 'test.LICENSE')
         file_path2 = posixpath.join(posixpath.dirname(test_file), 'test.NOTICE')
 
-        err_msg1 = u'Field license_file: Path %s not found' % file_path1
-        err_msg2 = u'Field notice_file: Path %s not found' % file_path2
+        err_msg1 = Error(CRITICAL, 'Field license_file: Path %s not found' % file_path1)
+        err_msg2 = Error(CRITICAL, 'Field notice_file: Path %s not found' % file_path2)
 
         expected_errors = [err_msg1, err_msg2]
-        errors = model.check_file_field_exist(a, test_file)
-        assert expected_errors == errors
+        assert expected_errors == a.errors
 
         assert {'test.LICENSE': None} == a.license_file.value
         assert {'test.NOTICE': None} == a.notice_file.value
@@ -565,6 +564,13 @@ this software and releases the component to Public Domain.
             Error(INFO, u'Field Mat\xedas is not a supported field and is ignored.')]
         assert expected == result
 
+    def test_About_invalid_boolean_value(self):
+        test_file = get_test_loc('parse/invalid_boolean.about')
+        a = model.About(test_file)
+        result = a.errors
+        expected_msg = "Field modified: Invalid flag value: 'blah'"
+        assert expected_msg in a.errors[0].message
+
     def test_About_contains_about_file_path(self):
         test_file = get_test_loc('parse/complete/about.ABOUT')
         a = model.About(test_file, about_file_path='complete/about.ABOUT')
@@ -587,45 +593,6 @@ this software and releases the component to Public Domain.
         assert a.dumps(True) == b.dumps(True)
         assert a == b
 
-    def test_About_dumps_does_not_transform_strings_in_lists(self):
-        test_file = get_test_loc('dumps/complete2/about.ABOUT')
-        a = model.About(test_file, about_file_path='complete2/about.ABOUT')
-        expected = u'''about_resource: .
-name: AboutCode
-about_resource_path:
-version: 0.11.0
-download_url:
-description: AboutCode is a tool to process ABOUT files. An ABOUT file is a file.
-homepage_url: http://dejacode.org
-notes:
-license: apache-2.0
-license_expression: apache-2.0
-license_name:
-license_file: apache-2.0.LICENSE
-license_url:
-copyright: Copyright (c) 2013-2014 nexB Inc.
-notice_file: NOTICE
-notice_url:
-redistribute:
-attribute:
-track_change:
-modified:
-changelog_file:
-owner: nexB Inc.
-owner_url:
-contact:
-author: Jillian Daguil, Chin Yeung Li, Philippe Ombredanne, Thomas Druez
-vcs_tool: git
-vcs_repository: https://github.com/dejacode/about-code-tool.git
-vcs_path:
-vcs_tag:
-vcs_branch:
-vcs_revision:
-checksum_md5:
-checksum_sha1:
-spec_version:
-'''
-        assert expected == a.dumps(with_absent=True)
 
     def test_About_same_attribution(self):
         base_dir = 'some_dir'
@@ -673,8 +640,8 @@ spec_version:
             'description',
             'homepage_url',
             'notes',
-            'license',
             'license_expression',
+            'license_key',
             'license_name',
             'license_file',
             'license_url',
@@ -683,13 +650,14 @@ spec_version:
             'notice_url',
             'redistribute',
             'attribute',
-            'track_change',
+            'track_changes',
             'modified',
             'changelog_file',
             'owner',
             'owner_url',
             'contact',
             'author',
+            'author_file',
             'vcs_tool',
             'vcs_repository',
             'vcs_path',
@@ -698,6 +666,7 @@ spec_version:
             'vcs_revision',
             'checksum_md5',
             'checksum_sha1',
+            'checksum_sha256',
             'spec_version',
             'f',
             'g']
@@ -738,27 +707,35 @@ class SerializationTest(unittest.TestCase):
         assert [] == a.errors
 
         expected = u'''about_resource: .
-name: AboutCode
-version: 0.11.0
-description: |
+author:
+    - Jillian Daguil
+    - Chin Yeung Li
+    - Philippe Ombredanne
+    - Thomas Druez
+copyright: Copyright (c) 2013-2014 nexB Inc.
+description: |-
     AboutCode is a tool
     to process ABOUT files.
     An ABOUT file is a file.
-homepage_url: http://dejacode.org
-license: apache-2.0
+homepage_url:
+    - http://dejacode.org
 license_expression: apache-2.0
-license_file: apache-2.0.LICENSE
-copyright: Copyright (c) 2013-2014 nexB Inc.
+licenses:
+    -   file: apache-2.0.LICENSE
+        key: apache-2.0
+name: AboutCode
 notice_file: NOTICE
-owner: nexB Inc.
-author: Jillian Daguil, Chin Yeung Li, Philippe Ombredanne, Thomas Druez
-vcs_tool: git
+owner:
+    - nexB Inc.
 vcs_repository: https://github.com/dejacode/about-code-tool.git
+vcs_tool: git
+version: 0.11.0
 '''
         result = a.dumps()
         assert expected == result
 
-    def test_About_dumps_all_fields_if_not_present_with_absent_True(self):
+    # We do not support with_absent and with_empty staring in version 3.2.0.
+    def FAILING_test_About_dumps_all_fields_if_not_present_with_absent_True(self):
         test_file = get_test_loc('parse/complete2/about.ABOUT')
         a = model.About(test_file)
         expected_error = [
@@ -783,7 +760,7 @@ notice_file:
 notice_url:
 redistribute:
 attribute:
-track_change:
+track_changes:
 modified:
 changelog_file:
 owner:
@@ -798,6 +775,7 @@ vcs_branch:
 vcs_revision:
 checksum_md5:
 checksum_sha1:
+checksum_sha256:
 spec_version:
 '''
         result = a.dumps(with_absent=True)
@@ -818,6 +796,27 @@ version: 0.11.0
         result = a.dumps(with_absent=False)
         assert set(expected) == set(result)
 
+    def test_About_dumps_with_different_boolean_value(self):
+        test_file = get_test_loc('parse/complete2/about2.ABOUT')
+        a = model.About(test_file)
+        expected_error_msg = "Field track_changes: Invalid flag value: 'blah' is not one of"
+        assert len(a.errors) == 1
+        assert expected_error_msg in a.errors[0].message
+
+        expected = u'''about_resource: .
+
+name: AboutCode
+version: 0.11.0
+
+redistribute: no
+attribute: yes
+modified: yes
+'''
+
+        result = a.dumps()
+        assert set(expected) == set(result)
+
+
     def test_About_dumps_does_not_dump_present__empty_with_absent_False(self):
         test_file = get_test_loc('parse/complete2/about.ABOUT')
         a = model.About(test_file)
@@ -843,7 +842,8 @@ version: 0.11.0
         result = as_dict[model.About.about_file_path_attr]
         assert expected == result
 
-    def test_About_as_dict_with_empty(self):
+    # The with_present and empty is no longer supported.
+    def FAILING_test_About_as_dict_with_empty(self):
         test_file = get_test_loc('as_dict/about.ABOUT')
         a = model.About(test_file, about_file_path='complete/about.ABOUT')
         expected_errors = [
@@ -855,7 +855,7 @@ version: 0.11.0
                     'author': u'',
                     'copyright': u'Copyright (c) 2013-2014 nexB Inc.',
                     'description': u'AboutCode is a tool\nfor files.',
-                    'license': u'apache-2.0',
+                    'license_key': u'apache-2.0',
                     'license_expression': u'apache-2.0',
                     'name': u'AboutCode',
                     'owner': u'nexB Inc.'}
@@ -865,7 +865,8 @@ version: 0.11.0
         # FIXME: why converting back to dict?
         assert expected == dict(result)
 
-    def test_About_as_dict_with_present(self):
+    # The with_present and empty is no longer supported.
+    def FAILING_test_About_as_dict_with_present(self):
         test_file = get_test_loc('as_dict/about.ABOUT')
         a = model.About(test_file, about_file_path='complete/about.ABOUT')
         expected_errors = [
@@ -876,16 +877,18 @@ version: 0.11.0
         expected = {'about_resource': u'.',
                     'about_resource_path': u'',
                     'author': u'',
+                    'author_file': u'',
                     'attribute': u'',
                     'changelog_file': u'',
                     'checksum_md5': u'',
                     'checksum_sha1': u'',
+                    'checksum_sha256': u'',
                     'contact': u'',
                     'copyright': u'Copyright (c) 2013-2014 nexB Inc.',
                     'description': u'AboutCode is a tool\nfor files.',
                     'download_url': u'',
                     'homepage_url': u'',
-                    'license': u'apache-2.0',
+                    'license_key': u'apache-2.0',
                     'license_expression': u'apache-2.0',
                     'license_file': u'',
                     'license_name': u'',
@@ -899,7 +902,7 @@ version: 0.11.0
                     'owner_url': u'',
                     'redistribute': u'',
                     'spec_version': u'',
-                    'track_change': u'',
+                    'track_changes': u'',
                     'vcs_branch': u'',
                     'vcs_path': u'',
                     'vcs_repository': u'',
@@ -921,47 +924,19 @@ version: 0.11.0
             Error(INFO, u'Field custom_empty is not a supported field and is ignored.'),
             Error(WARNING, u'Field author is present but empty')]
         assert expected_errors == a.errors
-        expected = {'about_resource': u'.',
+        expected = {'about_resource': [u'.'],
                     'copyright': u'Copyright (c) 2013-2014 nexB Inc.',
                     'description': u'AboutCode is a tool\nfor files.',
-                    'license': u'apache-2.0',
+                    'license_key': [u'apache-2.0'],
                     'license_expression': u'apache-2.0',
                     'name': u'AboutCode',
-                    'owner': u'nexB Inc.'}
+                    'owner': [u'nexB Inc.']}
         result = a.as_dict(with_paths=False,
                            with_empty=False,
                            with_absent=False)
         # FIXME: why converting back to dict?
         assert expected == dict(result)
 
-    def test_loads_dumps_is_idempotent(self):
-        test1 = u'''about_resource: .
-name: AboutCode
-version: 0.11.0
-copyright: |
-    multi
-    line
-'''
-
-        test2 = u'''about_resource: .
-name: AboutCode
-version: 0.11.0
-copyright: >
-    multi
-    line
-'''
-        a = model.About()
-        base_dir = 'some_dir'
-        a.loads(test1, base_dir)
-        dumped = a.dumps(with_absent=False, with_empty=False)
-        assert test1 == dumped
-
-        b = model.About()
-        base_dir = 'some_dir'
-        b.loads(test2, base_dir)
-        dumped = b.dumps(with_absent=False, with_empty=False)
-        assert test2 == dumped
-
     def test_load_dump_is_idempotent(self):
         test_file = get_test_loc('load/this.ABOUT')
         a = model.About()
@@ -1009,13 +984,13 @@ copyright: >
                  'name': u'AboutCode',
                  'owner': u'nexB Inc.'}
 
-        expected = {'about_resource': u'.',
+        expected = {'about_resource': [u'.'],
                  'author': u'',
                  'copyright': u'Copyright (c) 2013-2014 nexB Inc.',
                  'description': u'AboutCode is a tool\nfor files.',
                  'license_expression': u'apache-2.0',
                  'name': u'AboutCode',
-                 'owner': u'nexB Inc.'}
+                 'owner': [u'nexB Inc.']}
 
         a = model.About()
         base_dir = 'some_dir'
@@ -1025,18 +1000,15 @@ copyright: >
         assert expected == dict(as_dict)
 
     def test_load_dict_handles_field_validation_correctly(self):
-        test = {u'about_resource': u'.',
-                u'about_resource_path': u'.',
+        test = {u'about_resource': [u'.'],
                 u'attribute': u'yes',
-                u'author': u'Jillian Daguil, Chin Yeung Li, Philippe Ombredanne, Thomas Druez',
+                u'author': [u'Jillian Daguil, Chin Yeung Li, Philippe Ombredanne, Thomas Druez'],
                 u'copyright': u'Copyright (c) 2013-2014 nexB Inc.',
                 u'description': u'AboutCode is a tool to process ABOUT files. An ABOUT file is a file.',
-                u'homepage_url': u'http://dejacode.org',
+                u'homepage_url': [u'http://dejacode.org'],
                 u'license_expression': u'apache-2.0',
-                u'license_file': u'apache-2.0.LICENSE',
                 u'name': u'AboutCode',
-                u'notice_file': u'NOTICE',
-                u'owner': u'nexB Inc.',
+                u'owner': [u'nexB Inc.'],
                 u'vcs_repository': u'https://github.com/dejacode/about-code-tool.git',
                 u'vcs_tool': u'git',
                 u'version': u'0.11.0'}
@@ -1069,7 +1041,6 @@ copyright: >
         expected = get_test_loc('load/expected.json')
         check_json(expected, result)
 
-
 class CollectorTest(unittest.TestCase):
 
     def test_collect_inventory_in_directory_with_correct_about_file_path(self):
@@ -1164,18 +1135,36 @@ class CollectorTest(unittest.TestCase):
         test_loc = get_test_loc('parse/name_mapping_test.ABOUT')
         mapping = True
         errors, abouts = model.collect_inventory(test_loc, mapping)
-        expected_msg = 'Field resource is a custom field'
-        assert len(errors) == 1
-        assert expected_msg in errors[0].message
+        expected_msg1 = 'Field resource is a custom field'
+        expected_msg2 = 'Field custom_mapping is not a supported field and is not defined in the mapping file. This field is ignored.'
+        assert len(errors) == 2
+        assert expected_msg1 in errors[0].message
+        assert expected_msg2 in errors[1].message
         # The not supported 'resource' value is collected
         assert abouts[0].resource.value
 
+    def test_collect_inventory_with_custom_mapping(self):
+        test_loc = get_test_loc('parse/name_mapping_test.ABOUT')
+        mapping = False
+        mapping_file = get_test_loc('custom-mapping-file/mapping.config')
+        errors, abouts = model.collect_inventory(test_loc, mapping, mapping_file)
+        expected_msg1 = 'Field resource is a custom field'
+        expected_msg2 = 'Field custom_mapping is a custom field'
+        assert len(errors) == 2
+        assert expected_msg1 in errors[0].message
+        assert expected_msg2 in errors[1].message
+        # The not supported 'resource' value is collected
+        assert abouts[0].resource.value
+        assert abouts[0].custom_mapping.value
+
     def test_collect_inventory_without_mapping(self):
         test_loc = get_test_loc('parse/name_mapping_test.ABOUT')
         errors, abouts = model.collect_inventory(test_loc)
-        expected_msg = 'Field resource is not a supported field and is ignored.'
-        assert len(errors) == 1
-        assert expected_msg in errors[0].message
+        expected_msg1 = 'Field resource is not a supported field and is ignored.'
+        expected_msg2 = 'Field custom_mapping is not a supported field and is ignored.'
+        assert len(errors) == 2
+        assert expected_msg1 in errors[0].message
+        assert expected_msg2 in errors[1].message
 
     def test_parse_license_expression(self):
         spec_char, returned_lic = model.parse_license_expression(u'mit or apache-2.0')
